local project_tbl, check_number = ...

local handler = {
	["commands"] = function(data)
		print("\127FFFFFF"..project_tbl.help["commands"])
	end,
	
	["pos"] = function(data)
		if (project_tbl.hasmdi) then return end

		local x, y = data[1], data[2]
		local container = project_tbl.display_container
		if (check_number(x) and check_number(y)) then
			if (x:match("^[+-]")) then x = container.cx + x end
			if (y:match("^[+-]")) then y = container.cy + y end
-- 			container.cx, container.cy = tonumber(x), tonumber(y)
-- 			HUD.cboxlayer.REFRESH = "YES"
			project_tbl.x = x
			project_tbl.y = y
			project_tbl:SaveSettings()
			project_tbl:RebuildDisplay()

		else
			if (data[1] == "reset") then
				project_tbl.x = nil
				project_tbl.y = nil
				print("\127FFFFFFTargetlist position has been reset to default.")
				project_tbl:SaveSettings()
				project_tbl:RebuildDisplay()
			else
				local pos = container.cx.."x"..container.cy -- use the container positions since nil can't be concatenated
				if (not project_tbl.x) then pos = "unset (actual "..pos..")" end
				print("\127FFFFFFTargetlist position is "..pos)
			end
		end
	end,
	
	["fontscale"] = function(data)
		local newfontscale = check_number(data[1], project_tbl.defaults.fontscale.Lval, project_tbl.defaults.fontscale.Uval, project_tbl.defaults.fontscale.dp)
		if (newfontscale) then
			newfontscale = newfontscale < 0.1 and 0.1 or newfontscale
			project_tbl.fontscale = newfontscale
			print("\127FFFFFFFont size has been changed to "..newfontscale..".")
			project_tbl:SaveSettings()
			project_tbl:RebuildDisplay()
		else
			if (data[1] == "reset") then
				project_tbl.fontscale = project_tbl.defaults.fontscale.val
				print("\127FFFFFFThe font size has been been reset to "..project_tbl.fontscale..".")
				project_tbl:SaveSettings()
				project_tbl:RebuildDisplay()
			else
				print("\127FFFFFFThe current font size is "..project_tbl.fontscale..".")
			end
		end
	end,
	
	["help"] = function(data)
		local word = table.remove(data, 1)
		if (word) then
			word = string.lower(tostring(word))
			if (type(project_tbl.help[word]) == "string") then
				print("\127FFFFFF"..project_tbl.help[word])
			end

		else
			print("\127FFFFFF"..project_tbl.help["help"])
		end
	end,
              
	["highlight"] = function(data)
		local r = check_number(data[1], project_tbl.defaults.target_color.Lval, project_tbl.defaults.target_color.Uval)
		local g = check_number(data[2], project_tbl.defaults.target_color.Lval, project_tbl.defaults.target_color.Uval)
		local b = check_number(data[3], project_tbl.defaults.target_color.Lval, project_tbl.defaults.target_color.Uval)
		local a = check_number(data[4], project_tbl.defaults.target_color.Lval, project_tbl.defaults.target_color.Uval)
		if (r and g and b) then
			a = a or 65
			project_tbl.current_target_color = string.format("%d %d %d %d *", r, g, b, a)
			print("\127FFFFFFThe highlight color has been changed to "..project_tbl.current_target_color:match("(.*) %*"))
			project_tbl:SaveSettings()
		else
			if (data[1] == "reset") then
				project_tbl.current_target_color = project_tbl.defaults.target_color.val
				print("\127FFFFFFThe highlight color has been reset to "..project_tbl.current_target_color:match("(.*) %*"))
				project_tbl:SaveSettings()
			else
				print("\127FFFFFFThe current highlight color is "..project_tbl.current_target_color:match("(.*) %*"))
			end
		end
	end,
	
	["rows"] = function(data)
		local newrows = check_number(data[1], project_tbl.defaults.list_count.Lval)
		if (newrows) then
			project_tbl.list_count = newrows
			print("\127FFFFFFThe number of rows has been changed to "..newrows..".")
			project_tbl:SaveSettings()
			project_tbl:RebuildDisplay()
		else
			if (data[1] == "reset") then
				project_tbl.list_count = project_tbl.defaults.list_count.val
				print("\127FFFFFFThe number of rows has been been reset to "..project_tbl.list_count)
				project_tbl:SaveSettings()
				project_tbl:RebuildDisplay()
			else
				print("\127FFFFFFThe current number of rows is "..project_tbl.list_count)
			end
		end
	end,
	
	["width"] = function(data)
		local newwidth = check_number(data[1], project_tbl.defaults.name_width.Lval)
		if (newwidth) then
			project_tbl.name_width = newwidth
			print("\127FFFFFFName width has been changed to "..newwidth)
			project_tbl:SaveSettings()
			project_tbl:RebuildDisplay()

		else
			if (data[1] == "reset") then
				project_tbl.name_width = nil
				print("\127FFFFFFName width has been reset.")
				project_tbl:SaveSettings()
				project_tbl:RebuildDisplay()
			else
				local width = project_tbl.display[1][2][3][2].w
				if (not project_tbl.name_width) then width = "unset (actual "..width..")" end
				print("\127FFFFFFThe current name width is "..width..".")
			end
		end
	end,

	["options"] = function()
		ShowDialog(project_tbl.optionsdlg)
	end,
}

local function cmd_processor(_, data)
	if (not data) then
		handler["options"]()
-- 		print("\127FFFFFF"..project_tbl.help["help"])
		return
	end
	local cmd = string.lower(table.remove(data, 1))
	if (type(handler[cmd]) == "function") then handler[cmd](data) end
end

local function cycle_filter(_, data)
	if (data) then data = tonumber(data[1]) % 4 end
	project_tbl.filters:SetHighlight(project_tbl.filter + 1, false)
	project_tbl.filter = data or ((project_tbl.filter + 1) % 4)
	project_tbl.filters:SetHighlight(project_tbl.filter + 1, true)
	gkini.WriteInt(project_tbl.config_name, "Filter", project_tbl.filter)
end

local function cycle_faction_filter()
	project_tbl.faction_filter = (project_tbl.faction_filter + 1) % 4
	project_tbl.filters:SetHighlight(project_tbl.filter + 1, true)
	
end

local function change_target(i)
	if (project_tbl.shiplist[i]) then
		radar.SetRadarSelection(project_tbl.shiplist[i].nodeid, project_tbl.shiplist[i].objectid)
	end
end

local function target_next()
	local i = project_tbl.current_target_listindex or 0
	i = (i + 1) % project_tbl.list_count
	if (i == 0) then i = project_tbl.list_count end
	if (i > #project_tbl.shiplist) then i = 1 end
	change_target(i)
end

local function target_prev()
	local i = project_tbl.current_target_listindex or 0
	i = (i - 1) % project_tbl.list_count
	if (i == 0) then i = project_tbl.list_count end
	if (i > #project_tbl.shiplist) then i = #project_tbl.shiplist end
	change_target(i)
end

local verb = {[true] = "HIDING", [false] = "SHOWING"} -- reversed logic because the variable is hide_turrets
local verb_color = {[true] = "\127FF0000", [false] = "\12700FF00"}
local function toggleturrets()
	project_tbl.hide_turrets = not project_tbl.hide_turrets
	gkini.WriteInt(project_tbl.config_name, "Hide Turrets", project_tbl.hide_turrets and 1 or 0)
	print(string.format("%sTargetList is now %s%s%s turrets.", "\127FFFFFF", verb_color[project_tbl.hide_turrets], verb[project_tbl.hide_turrets], "\127FFFFFF"))
end

local verb = {[true] = "NPC ships", [false] = "NPC names"}
local function togglenpcships()
	project_tbl.show_npc_ships = not project_tbl.show_npc_ships
	gkini.WriteInt(project_tbl.config_name, project_tbl.defaults.show_npc_ships.name, project_tbl.show_npc_ships and 1 or 0)
	print(string.format("%sTargetList is now showing %s%s%s.", "\127FFFFFF", verb_color[not project_tbl.show_npc_ships], verb[project_tbl.show_npc_ships], "\127FFFFFF"))
end

local verb = {[true] = "INVERTED", [false] = "NORMAL"}
local function invert_faction_filter()
	project_tbl.invert_faction_filter = not project_tbl.invert_faction_filter
	gkini.WriteInt(project_tbl.config_name, "Invert Faction Filter", project_tbl.invert_faction_filter and 1 or 0)
	print(string.format("%sTargetList faction filter is now %s%s%s.", "\127FFFFFF", verb_color[project_tbl.invert_faction_filter], verb[project_tbl.invert_faction_filter], "\127FFFFFF"))
	iup.GetBrother(project_tbl.filters).bgcolor = "255 0 0 "..(project_tbl.invert_faction_filter and 35 or 0)
end

local function set_key_binds()
	for i = 1, project_tbl.list_count < 10 and project_tbl.list_count or 10 do 
		gkinterface.GKProcessCommand("bind "..(i % 10).." target"..i)
	end
	gkinterface.GKProcessCommand("bind - cyclefilter")
	gkinterface.GKProcessCommand("bind = cyclefactionfilter")
end

for i = 1, project_tbl.list_count < 10 and project_tbl.list_count or 10 do 
	RegisterUserCommand("target"..i, change_target, i)
end
RegisterUserCommand("cyclefilter", cycle_filter)
RegisterUserCommand("cyclefactionfilter", cycle_faction_filter)
RegisterUserCommand("invertfactionfilter", invert_faction_filter)
RegisterUserCommand("targetnext", target_next)
RegisterUserCommand("targetprev", target_prev)
RegisterUserCommand("toggleturrets", toggleturrets)
RegisterUserCommand("togglenpcships", togglenpcships)
RegisterUserCommand("settargetbinds", set_key_binds)
RegisterUserCommand("targetlist", cmd_processor)

return change_target
